package javax.swing.plaf.basic;

import javax.swing.plaf.ComponentInputMapUIResource;
import javax.swing.MenuElement;
import javax.swing.MenuSelectionManager;
import java.awt.Rectangle;
import javax.swing.JPopupMenu;
import java.awt.Component;
import javax.swing.JButton;
import java.awt.Point;
import java.awt.KeyboardFocusManager;
import java.awt.event.ActionEvent;
import sun.swing.UIAction;
import java.beans.PropertyChangeEvent;
import javax.swing.ComponentInputMap;
import sun.swing.DefaultLookup;
import javax.swing.InputMap;
import javax.swing.SwingUtilities;
import javax.swing.LookAndFeel;
import javax.swing.JRootPane;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.RootPaneUI;

public class BasicRootPaneUI extends RootPaneUI implements PropertyChangeListener
{
    private static RootPaneUI rootPaneUI;
    
    public BasicRootPaneUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        return BasicRootPaneUI.rootPaneUI;
    }
    
    @Override
    public void installUI(JComponent component) {
        this.installDefaults((JRootPane)component);
        this.installComponents((JRootPane)component);
        this.installListeners((JRootPane)component);
        this.installKeyboardActions((JRootPane)component);
    }
    
    @Override
    public void uninstallUI(JComponent component) {
        this.uninstallDefaults((JRootPane)component);
        this.uninstallComponents((JRootPane)component);
        this.uninstallListeners((JRootPane)component);
        this.uninstallKeyboardActions((JRootPane)component);
    }
    
    protected void installDefaults(JRootPane c) {
        LookAndFeel.installProperty(c, "opaque", Boolean.FALSE);
    }
    
    protected void installComponents(JRootPane rootPane) {
    }
    
    protected void installListeners(JRootPane rootPane) {
        rootPane.addPropertyChangeListener(this);
    }
    
    protected void installKeyboardActions(JRootPane rootPane) {
        SwingUtilities.replaceUIInputMap(rootPane, 2, this.getInputMap(2, rootPane));
        SwingUtilities.replaceUIInputMap(rootPane, 1, this.getInputMap(1, rootPane));
        LazyActionMap.installLazyActionMap(rootPane, BasicRootPaneUI.class, "RootPane.actionMap");
        this.updateDefaultButtonBindings(rootPane);
    }
    
    protected void uninstallDefaults(JRootPane rootPane) {
    }
    
    protected void uninstallComponents(JRootPane rootPane) {
    }
    
    protected void uninstallListeners(JRootPane rootPane) {
        rootPane.removePropertyChangeListener(this);
    }
    
    protected void uninstallKeyboardActions(JRootPane rootPane) {
        SwingUtilities.replaceUIInputMap(rootPane, 2, null);
        SwingUtilities.replaceUIActionMap(rootPane, null);
    }
    
    InputMap getInputMap(int n, JComponent c) {
        if (n == 1) {
            return (InputMap)DefaultLookup.get(c, this, "RootPane.ancestorInputMap");
        }
        if (n == 2) {
            return this.createInputMap(n, c);
        }
        return null;
    }
    
    ComponentInputMap createInputMap(int n, JComponent component) {
        return new RootPaneInputMap(component);
    }
    
    static void loadActionMap(LazyActionMap lazyActionMap) {
        lazyActionMap.put(new BasicRootPaneUI.Actions("press"));
        lazyActionMap.put(new BasicRootPaneUI.Actions("release"));
        lazyActionMap.put(new BasicRootPaneUI.Actions("postPopup"));
    }
    
    void updateDefaultButtonBindings(JRootPane rootPane) {
        InputMap retMap;
        for (retMap = SwingUtilities.getUIInputMap(rootPane, 2); retMap != null && !(retMap instanceof BasicRootPaneUI.RootPaneInputMap); retMap = retMap.getParent()) {}
        if (retMap != null) {
            retMap.clear();
            if (rootPane.getDefaultButton() != null) {
                Object[] keys = (Object[])(Object[])DefaultLookup.get(rootPane, this, "RootPane.defaultButtonWindowKeyBindings");
                if (keys != null) {
                    LookAndFeel.loadKeyBindings(retMap, keys);
                }
            }
        }
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        if (propertyChangeEvent.getPropertyName().equals("defaultButton")) {
            JRootPane rootPane = (JRootPane)propertyChangeEvent.getSource();
            this.updateDefaultButtonBindings(rootPane);
            if (rootPane.getClientProperty("temporaryDefaultButton") == null) {
                rootPane.putClientProperty("initialDefaultButton", propertyChangeEvent.getNewValue());
            }
        }
    }
    
    static {
        BasicRootPaneUI.rootPaneUI = new BasicRootPaneUI();
    }
    
    static class Actions extends UIAction
    {
        public static final String PRESS = "press";
        public static final String RELEASE = "release";
        public static final String POST_POPUP = "postPopup";
        
        Actions(String name) {
            super(name);
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JRootPane rootPane = (JRootPane)actionEvent.getSource();
            JButton defaultButton = rootPane.getDefaultButton();
            String name = this.getName();
            if (name == "postPopup") {
                Component focusOwner = KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner();
                if ((focusOwner instanceof JComponent)) {
                    JComponent component = (JComponent)focusOwner;
                    JPopupMenu componentPopupMenu = component.getComponentPopupMenu();
                    if (componentPopupMenu != null) {
                        Point popupLocation = component.getPopupLocation(null);
                        if (popupLocation == null) {
                            Rectangle visibleRect = component.getVisibleRect();
                            popupLocation = new Point(visibleRect.x + visibleRect.width / 2, visibleRect.y + visibleRect.height / 2);
                        }
                        componentPopupMenu.show(focusOwner, popupLocation.x, popupLocation.y);
                    }
                }
            }
            else if (defaultButton != null && SwingUtilities.getRootPane(defaultButton) == rootPane && name == "press") {
                defaultButton.doClick(20);
            }
        }
        
        public boolean isEnabled(Object o) {
            if (this.getName() == "postPopup") {
                MenuElement[] selectedPath = MenuSelectionManager.defaultManager().getSelectedPath();
                if (selectedPath != null && selectedPath.length != 0) {
                    return false;
                }
                Component focusOwner = KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner();
                return focusOwner instanceof JComponent && ((JComponent)focusOwner).getComponentPopupMenu() != null;
            }
            else {
                if (o != null && o instanceof JRootPane) {
                    JButton defaultButton = ((JRootPane)o).getDefaultButton();
                    return defaultButton != null && defaultButton.getModel().isEnabled();
                }
                return true;
            }
        }
    }
    
    private static class RootPaneInputMap extends ComponentInputMapUIResource
    {
        public RootPaneInputMap(JComponent component) {
            super(component);
        }
    }
}
